#include <iostream>
#include <iomanip>
#include <fstream>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string>
#include "rof.h"
#include "cell.h"

#ifdef _OPENMP
#include <omp.h> // OpenMP
#endif

using namespace std;

//֐̐錾difinition of the function
int makeBasinList(int row, int col, class Cell** object, int*& upperGridNum);
void read2DValueAsc(ROF** fROF, int n_year, int n_month, int ROFnum_row, int ROFnum_col);
void read2DValueGrads(ROF** fROF, int n_year, int n_month, int ROFnum_row, int ROFnum_col);

int main()
{

	cout << "Reading parameters for runoff simulation." << endl;
	ifstream finParam("./param.dat");
	if(!finParam){
		cout << "param.dat cannot open." << endl;
		return 1;
	}

	// vZ̈ simulation range
	cout << "1) Reading parameters for study area." << endl;
	int wCol, wRow, wTopoScale;
	float wSWLON, wSWLAT;

	finParam >> wTopoScale;
	cout << "  Topography scale : " << wTopoScale << endl;

	finParam >> wCol; 
	finParam >> wRow; 
	cout << "  Number of col : " << wCol << "  Number of row : " << wRow << endl;

	finParam >> wSWLON; 
	finParam >> wSWLAT;
	cout << "  Sorth West corner Longitude (degree) : " << wSWLON 
		 << " Latitude (deg) : " << wSWLAT << endl;

	const int TopoScale = wTopoScale;  // Topography scale
	const int Col = wCol;     // Ώۗ̈̍sB120 ox 1 xɑΉB
	const int Row = wRow;     // Ώۗ̈̍sB120 ܓx 1 xɑΉB
	const float SWLON = wSWLON; // South west corner of longitude (degree)
	const float SWLAT = wSWLAT; // South west corner of latiitude (degree)

	// ̓f[^̈ input data range
	cout << "2) Reading parameters for input data." << endl;

	int wROFnum_row, wROFnum_col;
	finParam >> wROFnum_col;
	finParam >> wROFnum_row;
	cout << "  Number of col : " << wROFnum_col 
		 << "  Number of row : " << wROFnum_row << endl;

	int wpO_x, wpO_y;
	finParam >> wpO_x;
	finParam >> wpO_y;
	cout << "  Origin column number : " << wpO_x 
		 << " row number : " << wpO_y << endl;

	float wDelta_x, wDelta_y;
	finParam >> wDelta_x;
	finParam >> wDelta_y;
	cout << "  Longitudinal resolution : " << wDelta_x 
		 << "  Latitudinal resolution : " << wDelta_y << endl;

	float wROF_WLON;
	finParam >> wROF_WLON; 
	cout << "  The longitude of the center of input grid at the southwest corner : " << wROF_WLON << endl;

	float wROF_WLAT;
	finParam >> wROF_WLAT; 
	cout << "  The latitude of the center of input grid at the southwest corner : " << wROF_WLAT << endl;

	int wInputData_Time_Step;
	finParam >> wInputData_Time_Step; 
	cout << "  time interval of input data [sec] : " 
		 << wInputData_Time_Step << endl;
													
	const int ROFnum_col = wROFnum_col;
	const int ROFnum_row = wROFnum_row; 
	const int pO_x = wpO_x;
	const int pO_y = wpO_y;
	const float Delta_x = wDelta_x;
	const float Delta_y = wDelta_y;
	const float ROF_WLON = wROF_WLON;
	const float ROF_WLAT = wROF_WLAT;
	const int InputData_Time_Step = wInputData_Time_Step;

	// vZ simulation term
	cout << "3) Reading parameters for simuration term." << endl;
	int wstartYear, wendYear, wstartMonth, wendMonth;
	finParam >> wstartYear;
	finParam >> wendYear;
	finParam >> wstartMonth;
	finParam >> wendMonth;
	cout << "  Start year : " << wstartYear << "  Start month : " << wstartMonth << endl;
	cout << "  End year : " << wendYear << "  End month : " << wendMonth << endl;

	const int startYear = wstartYear; // vZJnN start year
	const int endYear   = wendYear; // vZIN end year

	if (startYear > endYear) {
		cout << "Simulation year setting is wrong" << endl;
		exit(1);
	}

	const int startMonth = wstartMonth;   // vZJn start month
	const int endMonth   = wendMonth;  // vZI end month

	//  finite difference condition
	cout << "4) Reading parameters for finite difference condition." << endl;
	int wTime_Step, wNspace; double wTheta;
	finParam >> wTime_Step; cout << "  Finite difference time step [sec] : " << wTime_Step << endl;
	finParam >> wNspace; cout << "  Finite difference space step : " << wNspace << endl;
	finParam >> wTheta; cout << "  Finite difference weight : " << wTheta << endl;

	const int Time_Step = wTime_Step; // vZ̎ԊԊu [sec] similation time step [sec] */
	const int Nspace = wNspace; /* vZ̋ԕ number of spatial divisions */
	const double Theta = wTheta; 	/* d݌W (-) 0.5 < theta <= 1.0 */
	const int Ntime = InputData_Time_Step/Time_Step;  // ̓f[^ԊǔvZ

	// model parameter setting for runoff simulation vZ̃p^
	cout << "5) Reading parameters for kinematic wave model." << endl;
	double wMan_slope, wMan_channel; int wThreshold_basin;
	finParam >> wMan_slope; cout << "  Manning roughmess coefficient for slope [m-s unit] : " << wMan_slope << endl;
	finParam >> wMan_channel; cout << "  Manning roughmess coefficient for channel [m-s unit] : " << wMan_channel << endl;
	finParam >> wThreshold_basin; cout << "  Area to classify slope and channel : " << wThreshold_basin << endl;

	const double Man_slope   =  wMan_slope; // Ζʂł̑exW for slope
	const double Man_channel = wMan_channel; // ͓ł̑exW for river
	const int Threshold_basin = wThreshold_basin; // WObh臒libasin > threshold_basin ݂Obh͓ɂȂj

	/* initial condi */
	cout << "6) Reading itinial discharge for kinematic wave model." << endl;
	double winitialROF;
	finParam >> winitialROF; cout << "  Initial runoff height [mm/hr] : " << winitialROF << endl;
	const double intialROF = winitialROF; /* initial runoff height [mm/hr] */

	cout << endl;
	finParam.close();
	//--------------------- end parameter reading ------------------------------------------------


	//ϐ̒` dif-variables
	int iday, ihour, dt;
	int n, k;
	bool executeOrPass = false;
	//errno_t error;
	
	const int dayMonth[] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

	FILE *fCorrectedMed;
	float med;

	FILE *fDown;
	FILE *fBasin;
	FILE *fRiverNumber;

	float ROFlat;
	float ROFlon;
	char DischargeFileNamebin[256];
	int* upperGridNum;
	int numOfBasin;

	float damOutflow;

	// reading watershed numbers for simulation from riverNumList.txt 
        // vZ闬̔ԍ̐ݒ
	ifstream fin1("./topoData/riverNumList.dat");	
	//ifstream fin1("./topoData/riverNumList.txt");
	if(!fin1){
		cout << "riverNumList.dat cannot open.\n";
		return 1;
	}

	int size_riverNumber_cal=0;
	string ss;

	while (getline(fin1, ss)) { // count number of rivers.  ΏۂƂ͐쐔𒲂ׂ
		size_riverNumber_cal++; 
	}
	fin1.close();
	cout << "Number of rivers for simulation : " << size_riverNumber_cal << endl << endl;

	int* riverNumber_cal;
	riverNumber_cal = new int[size_riverNumber_cal];

	ifstream fin2("./topoData/riverNumList.dat");
	if(!fin2){
		cout << "riverNumList.dat cannot open.\n";
		return 1;
	}
	int iriver, icol, irow, ilat, ilon, iarea;
	float dlon, dlat;
	for(int i=0; i<size_riverNumber_cal; i++){ // setting river number for simulation
		fin2 >> iriver >> iarea >> icol >> irow >> ilon >> ilat >> dlon >> dlat;
		riverNumber_cal[i]=iriver;
	}
	fin2.close();


	// Memory assignment for grid cell of simulation
	class Cell** cellObject; 
	cellObject = new class Cell*[Row];
	for(int i=0; i<Row; i++){
		cellObject[i] = new class Cell[Col];
	}

	// Memory assignment for data cell of input data
	class ROF** rofObject;
	rofObject = new class ROF*[ROFnum_row];
	for(int i=0; i<ROFnum_row; i++){
		rofObject[i] = new class ROF[ROFnum_col];
	}

	// Set information for each runoff sumulation cell 

	// Set latitude and longitude information
	/*ifstream fin("./topoData/dataLatitude.dat");
	if(!fin){
		cout << "dataLatitude.dat cannot open.\n";
		return 1;
	}
	*/

	for(int i=0; i<ROFnum_row; i++){
		ROFlat = ROF_WLAT + Delta_y*i; // set latitude ܓx 
//		fin >> ROFlat;  // read latitude from dataLatitude.dat ܓx
		for(int j=0; j<ROFnum_col; j++){
//			ROFlon = ROF_WLON + 0.5*Delta_x + Delta_x*j; // set longitude ox 
			// ROFlon ɂ͓쐼[̃Obh̒Šoxݒ肷B
			ROFlon = ROF_WLON + Delta_x*j; // Setting grid center of the longitude ox. 
			// Delta_x = 0.18750: oʃf[^̌oxԊu[degree]
			rofObject[i][j].setInformation(&ROFlat, &ROFlon, i, j);
		}
	}

//	fin.close();

	//--- [͓ԁEoϐɒl set the sream informations 

	// set elevation data 
	//if(error = fopen_s(&fCorrectedMed, "./topoData/modDem.bin", "rb") != NULL) {
	if ((fCorrectedMed = fopen("./topoData/modDem.bin", "rb")) == NULL) {
		cout << "modDem.bin cannot open." << endl;
		exit(1);
	}

	for(int i=Row-1; i>=0; i--) { // DEM data arranges from south to north
		for(int j=0; j<Col; j++){
			fread(&med, sizeof(float), 1, fCorrectedMed);
			cellObject[i][j].setElevation(i, j, Row, Col, SWLAT, SWLON, TopoScale, med);
			cellObject[i][j].setNULL();		
		}
	}

	fclose(fCorrectedMed);

	// set flow direction, accumulated area and river number information 
	//if(error = fopen_s(&fDown, "./topoData/flowDir.bin", "rb") != NULL) {
	if ((fDown = fopen("./topoData/flowDir.bin", "rb")) == NULL) {
		cout << "down.bin cannot open." << endl;
		exit(1);
	}

	//if(error = fopen_s(&fBasin, "./topoData/flowAcc.bin", "rb") != NULL) {
	if ((fBasin = fopen("./topoData/flowAcc.bin", "rb")) == NULL) {
		cout << "basin.bin cannot open." << endl;
		exit(1);
	}

	//if(error = fopen_s(&fRiverNumber, "./topoData/riverNum.bin", "rb") != NULL) {
	if ((fRiverNumber = fopen("./topoData/riverNum.bin", "rb")) == NULL) {
		cout << "riverNumber.bin cannot open." << endl;
		exit(1);
	}

	for(int i=Row-1; i>=0; i--) { // Topo data arranges from south to north
		for(int j=0; j<Col; j++){

			fread(&cellObject[i][j].down, sizeof(int), 1, fDown);
			fread(&cellObject[i][j].basin, sizeof(int), 1, fBasin);
			fread(&cellObject[i][j].riverNumber, sizeof(int), 1, fRiverNumber);

			if(cellObject[i][j].riverNumber <= 0) {
				continue;
			}
			cellObject[i][j].setDownStreamGrid(i, j, cellObject);
		}
	}

	fclose(fDown);
	fclose(fBasin);
	fclose(fRiverNumber);

	/* upStreamGrid  riverNumber */
	for(int i=0; i<Row; i++){
		for(int j=0; j<Col; j++)	{
			if(cellObject[i][j].riverNumber <= 0 // out of study basins
				|| cellObject[i][j].gridKind < 0 // end of study area 
				|| cellObject[i][j].down==0) {
				continue;
			}
			//cout << i << ", " << j << endl;
			cellObject[i][j].setUpStreamGrid();
		}
	}

	/* numberOfUpStreamGrid */
	for(int i=0; i<Row; i++){
		for(int j=0; j<Col; j++){
			if(cellObject[i][j].riverNumber <= 0       // out of study basins
				|| cellObject[i][j].gridKind < 0) // end of study area 
			{
				continue;
			}
			cellObject[i][j].countNumberOfUpStreamGrid();
		}
	}

	/* ROFCell ԗʂ p^ set parameters */
	for(int i=0; i<Row; i++){
		for(int j=0; j<Col; j++){
			
			if(cellObject[i][j].riverNumber <= 0         // out of study basins
				|| cellObject[i][j].gridKind < 0) { // end of study area 
				continue;
			}

// n`ObhɂƂ߂̓Obhݒ肷B
// Cɋ߂n`Obhł́AƂ߂̓Obh̓f[^Ȃ\̂ŁA
// ̓f[^ǂ񂾌Ńf[^݂ł߂̓Obhݒ肷B
//			cellObject[i][j].setROFCell(rofObject, ROFnum_row, ROFnum_col);
			cellObject[i][j].Set_parameter(Man_slope, Man_channel, Threshold_basin, Nspace, Time_Step, Theta, Ntime, TopoScale);
			cellObject[i][j].initialize(intialROF, TopoScale);
		}
	}	

	/*** debugging to check model parameters *******************************
	ofstream fout("./output/checkParameter.txt");
	if(!fout){
		cout << "checkParameter.txt cannot open.\n";
		return 1;
	}
	fout << "longitude	latitude	basin	down	med	mG\n";
	ofstream fout1("./output/checkDown.txt");
	if(!fout1){
		cout << "checkDown.txt cannot open.\n";
		return 1;
	}

	for(int i=0; i<Row; i++){
		for(int j=0; j<Col; j++){
			for(int kk=0; kk<size_riverNumber_cal; kk++){
				if(cellObject[i][j].riverNumber==riverNumber_cal[kk]){
					fout << cellObject[i][j].riverNumber << "	" << cellObject[i][j].longitude 
						<< "	" << cellObject[i][j].latitude << "	" << cellObject[i][j].basin 
						<< "	" << cellObject[i][j].down << "	" << cellObject[i][j].med << "	" 
						<< cellObject[i][j].mG << endl;
				}
			}
			if(cellObject[i][j].gridKind < 0) { // end of study area 
				continue;
			}
			fout1 << setw(3) << cellObject[i][j].down;
		}
		fout1 << endl;
	}
	fout.close();
	fout1.close();
	*** debug end *************************************************************/


	//--- make basinList܂ ---
	numOfBasin = makeBasinList(Row, Col, cellObject, upperGridNum);
	cout << "*** Number of different kind of cell accumulattion number : " << numOfBasin << endl;


	// --- ovZ runoff simulation ---
	cout << "Simulation starts from " << startYear << "/" << startMonth 
		<< " to " << endYear << "/" << endMonth << "." << endl;

	// count total number of month for simulation vZ鑍JEg
	int totalMonth;
/*
	if ((endYear-startYear) == 0) {
		totalMonth = endMonth-startMonth+1;
	}
	else if ((endYear-startYear) == 1) {
		totalMonth = 13-startMonth+endMonth;
	}
	else if ((endYear-startYear) >= 2) {
		totalMonth = 13-startMonth+endMonth + (endYear-startYear)*12 ;
	}
*/

	// Corrected by S.J. Noh (17 Jan 2013)
	if ((endYear-startYear) < 0) {
	exit(1);
	}
	else {
		totalMonth = endMonth-startMonth+13+(endYear-startYear-1)*12 ;
	}

	cout << "Total simulation time is " << totalMonth << " months." << endl << endl; 
	
	int xROF;
#pragma omp parallel for private(xROF)
	// ʃf[^̏ initializing input data array.
	for(int yROF=0; yROF<ROFnum_row; yROF++){ 
		for(xROF=0; xROF<ROFnum_col; xROF++){
			rofObject[yROF][xROF].initialize();
		}
	}

	int iyear = startYear;
	int imonth = startMonth;

	int wyear, wmonth, wday;

	for(int itotalMonth=0; itotalMonth < totalMonth; itotalMonth++) { // [v̊Jn

		iyear = startYear + (startMonth+itotalMonth-1)/12;
		imonth = (startMonth+itotalMonth-1)%12 + 1;

		cout << iyear << "/" << imonth << " ... calculating" << endl;

		// ʃf[^̓ǂݍށBƂɓ̓f[^ZbgB
		// Use below 6 lines when reading one-dimensional input data 
        /*-------------------------
        #pragma omp parallel for private(xROF)
		for(int yROF=0; yROF<ROFnum_row; yROF++){ // latitude direction
			for(xROF=0; xROF<ROFnum_col; xROF++){ // longitude direction
				//rofObject[yROF][xROF].renewValue(imonth); // őO̍ŏĨf[^𓖌̔z[0]ɃZbg
				rofObject[yROF][xROF].readValue(iyear, imonth); // œ̓f[^Zbg
			}
		}
		*-------------------------*/

		// Use below line when reading two-dimensional input data 
		// 2 ksedge `̃eLXgf[^ǂݍޏꍇɗpB
		// read2DValueAsc(rofObject, iyear, imonth, ROFnum_row, ROFnum_col);

		// Reading monthly input data with Grads format
		read2DValueGrads(rofObject, iyear, imonth, ROFnum_row, ROFnum_col);

		/* ROFCell ԗʂ p^ set parameters */
		// n`ObhɂƂ߂̓Obhݒ肷B
		// Cɋ߂n`Obhł́AƂ߂̓Obh̓f[^Ȃ\̂ŁA
		// ̓f[^ǂ񂾌Ńf[^݂ł߂̓Obhݒ肷B
		if(itotalMonth==0) { // vZŏ̌̓̓f[^݂ē̓Obhݒ肷B
			for(int i=0; i<Row; i++){
				for(int j=0; j<Col; j++){
					
					if(cellObject[i][j].riverNumber <= 0         // out of study basins
						|| cellObject[i][j].gridKind < 0) { // end of study area 
						continue;
					}
					cellObject[i][j].setROFCell(rofObject, ROFnum_row, ROFnum_col);
				}
			}	
		}

		for(int ib = 0; ib < numOfBasin; ib++){
			//cout << upperGridNum[ib] << "	";

//#pragma omp parallel for private(DischargeFileName, fDischarge, iday, ihour, dt, n, executeOrPass, k, iObs)
#pragma omp parallel for private(iday, ihour, dt, n, executeOrPass, k)

			for(int m=0; m<Row; m++){ // latitude direction
				for(n=0; n<Col; n++){ // longitude direction

					if(cellObject[m][n].gridKind < 0) { // edge of study area 
						continue;
					}

					// w肵ɑObĥ݁Aȉ̑s 
					// Only catchments specified in 'riverNumberList.txt' is calculated.
					executeOrPass = false;

					for(k=0; k<size_riverNumber_cal; k++){
						if(cellObject[m][n].riverNumber == riverNumber_cal[k]){
							executeOrPass = true;
							break;
						}
					}

					if(executeOrPass == false) {
						continue;
					}

					if(cellObject[m][n].basin == upperGridNum[ib])
					{
						// runoff simulation for slope and channel for one month
						// cout << "ib = " << ib << " slp kind: col = " << n+1 << " row = " << Row - m << endl;
						for(iday=1; iday<=dayMonth[imonth-1]; iday++){ // day loop
							for(ihour=1; ihour<=24; ihour++){ // hour loop
								cellObject[m][n].getLateralRunoff(iday, ihour, TopoScale);
								cellObject[m][n].Calculate(iday, ihour, Ntime);
							}
						}
					}
				}
			} 			
		} // 1 month simulation is finished. ܂ł̃[v1̌vZIB

		ofstream FileDaily;
		//sprintf_s(DischargeFileNamebin, 256, "./output/%4d%02d_dailyDischarge.bin", iyear, imonth);
		sprintf(DischargeFileNamebin, "./output/%4d%02d_dailyDischarge.bin", iyear, imonth);
		cout << " " << DischargeFileNamebin << endl;
		FileDaily.open(DischargeFileNamebin, ios::out | ios::binary);
		
		ofstream FileHourlyMax;
		//sprintf_s(DischargeFileNamebin, 256, "./output/%4d%02d_maxDischargeInDay.bin", iyear, imonth);
		sprintf(DischargeFileNamebin, "./output/%4d%02d_maxDischargeInDay.bin", iyear, imonth);
		cout << " " << DischargeFileNamebin << endl;
		FileHourlyMax.open(DischargeFileNamebin, ios::out | ios::binary);

		/*--- for debugging ------------------------------------------------*/
		//ofstream FileTest;
		////sprintf_s(DischargeFileNamebin, 256, "./output/%4d%02d_Test_discharge.dat", iyear, imonth);
		//sprintf(DischargeFileNamebin, "./output/%4d%02d_Test_discharge.dat", iyear, imonth);
		//cout << " " << DischargeFileNamebin << endl;
		//FileTest.open(DischargeFileNamebin);
		/*--- for debugging------------------------------------------------ */

		float Nodata=-9999;		
		for(iday=0; iday<dayMonth[imonth-1]; iday++){
			float dailyFlow; //m3/hr
			float hourlyFlowMax; //m3/hr
				
			for(int m=Row-1; m >=0; m--){ // latitude direction from South for Grads format
				for(n=0; n<Col; n++){     // longitude direction from West

					if(cellObject[m][n].gridKind < 0){ // end of study area
						FileDaily.write((char*)&Nodata, sizeof(float));
						FileHourlyMax.write((char*)&Nodata, sizeof(float));
						continue;
					}
					// w肵ɑȂObhɂ false ݒ肷B 
					// 'false' is set to the grids of outside study area.
					executeOrPass = false;

					for(k=0; k<size_riverNumber_cal; k++){
						if(cellObject[m][n].riverNumber == riverNumber_cal[k]){
							executeOrPass = true;
							break;
						}
					}

					if(executeOrPass == false){ // ΏۊÕObhł Nodata ށB
						FileDaily.write((char*)&Nodata, sizeof(float));
						FileHourlyMax.write((char*)&Nodata, sizeof(float));
						continue;
					}
						
					dailyFlow = (float) cellObject[m][n].GetDailyFlow(iday);
					FileDaily.write((char*)&dailyFlow, sizeof(float));

					hourlyFlowMax = (float) cellObject[m][n].GetHourlyFlowMax(iday);
					FileHourlyMax.write((char*)&hourlyFlowMax, sizeof(float));
				}
			}
			/*--- File output for debugging by YT ------------------------------------------------------*/
			//int outCol = 55; // begin from 1 from east
			//int outRow = 2; // begin from 1 from south
			//outRow = Row -outRow + 1; // change row from north
			//dailyFlow = (float) cellObject[outRow-1][outCol-1].GetDailyFlow(iday, Ntime);
			//FileTest << iyear << " " << imonth << " " << iday+1 << " " << dailyFlow << endl;
			/*---debugging------------------------------------------------------------------------------*/
			/*
			float hourlyFlow; //m3/hr
			for (ihour=0; ihour < 24; ihour++) {
				hourlyFlow = (float) cellObject[outRow-1][outCol-1].GetHourlyFlow(iday, ihour, Ntime);
				FileTest << iyear << " " << imonth << " " << iday+1 << " " << ihour << " " << hourlyFlow << endl;
			}
			*/
		} 
			
		FileDaily.close();
		FileHourlyMax.close();
		/*--- debugging -------------------*/
		// FileTest.close();
		/*---------------------------------*/
	} // end of simulation loop. One loop corresponds to one month.

	for(int i=0; i<Row; i++){
		for(int j=0; j<Col; j++){
			if(cellObject[i][j].gridKind < 0 // end of study region
				|| cellObject[i][j].riverNumber <= 0){ // out of study basins
				continue;
			}
			cellObject[i][j].deleteQandA();
		}
	}

	for(int i=0; i<Row; i++){
		delete[] cellObject[i];
	}
	delete[] cellObject;

	for(int i=0; i<ROFnum_row; i++){
		delete[] rofObject[i];
	}
	delete[] rofObject;

	delete[] upperGridNum;

	return 0;
}


//	֐̒`

int makeBasinList(int row, int col, class Cell** object, int*& upperGridNum)
{
	int** basin;

	basin = new int*[row];
	for(int i=0; i<row; i++){
		
		basin[i] = new int[col];
	}


	//eIuWFNgbasinzbasinɂ
	for(int i=0; i<row; i++){
		for(int j=0; j<col; j++){
			basin[i][j] = object[i][j].basin;
		}
	}

	//ɕבւ
	int tmp;

	for(int i=0; i<row; i++){
		for(int j=0; j<col; j++){
			
			for(int jj=j+1; jj<col; jj++){
				
				if(basin[i][j] > basin[i][jj]){
					
					tmp = basin[i][j];
					basin[i][j] = basin[i][jj];
					basin[i][jj] = tmp;
				}
			}
			
			for(int ii=i+1; ii<row; ii++){
				for(int jj=0; jj<col; jj++){
					
					if(basin[i][j] > basin[ii][jj]){
						
						tmp = basin[i][j];
						basin[i][j] = basin[ii][jj];
						basin[ii][jj] = tmp;
					}
				}
			}
			
		}
	}

	//zbasin̗vfɏdĂȂ͉邩
	int num = 0;
	int dummy = 0;

	for(int i=0; i<row; i++){
		for(int j=0; j<col; j++){
			
			if(basin[i][j] > 0 && basin[i][j] != dummy){
				dummy = basin[i][j];
				num++;
			}
		}
	}

	//zupperGridNum̃m
	upperGridNum = new int[num];

	//zupperGridNumbasin̒lɓ
	int k=0;
	for(int i=0; i<row; i++){
		for(int j=0; j<col; j++){
			
			if(basin[i][j] > 0 && basin[i][j] != dummy){
				*(upperGridNum+k) = basin[i][j];
				dummy = basin[i][j];
				k++;
			}
		}
	}

	for(int i=0; i<row; i++){
		delete[] basin[i];
	}
	delete[] basin;

	return num;
}

void read2DValueGrads(ROF** fROF, int n_year, int n_month, int ROFnum_row, int ROFnum_col)
{
	// read Grads format binary two dimensional data. 

	char ROFfilename[256];
	
	//sprintf_s(ROFfilename, 256, "./input/%04d%02d_rain.bin", n_year, n_month);
	sprintf(ROFfilename, "./input/%04d%02d_rain.bin", n_year, n_month);
//	sprintf(ROFfilename, "./input/newrof_gl_spa2_Honshu2_%04d%02d.gad", n_year, n_month);
	cout << ROFfilename << endl;

	ifstream fin;
	
	fin.open(ROFfilename, ios::in | ios::binary);
	if(!fin){
		cout << "ROF file cannot open.\n";
		exit(1);
	}

	//Ǒ̖̃f[^ value[0][ihour] ɃZbg
	const int dayMonth[] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};	

	int c_month;
	c_month = n_month - 1;
	if(c_month == 0){
		c_month = c_month + 12;
	}

	// O̍ŏI̎ԗʂ0ڂ̎ԗʂɃZbg
	for(int i=0; i<ROFnum_row; i++){
		for(int j=0; j<ROFnum_col; j++){		
			for(int ihour=0; ihour<24; ihour++){
				fROF[i][j].value[0][ihour] = fROF[i][j].value[dayMonth[c_month-1]][ihour];
			}
		}
	}

	// ǂݍݎԒZk̂߁C1Pʎԕ̑Sf[^xɓǂ
	// 2016/11/10 T. Tanaka
	float rainAllCell[ROFnum_row*ROFnum_col];
	for(int iday=1; iday <= dayMonth[n_month-1]; iday++){  // day loop
		for(int ihour=0; ihour<24; ihour++){  // hour loop
			fin.read((char*)&rainAllCell, 4*ROFnum_row*ROFnum_col);
			for(int i=0; i<ROFnum_row; i++){  // 1st row data is the southest row!
				for(int j=0; j<ROFnum_col; j++){  // 1st column is the westest column!
					fROF[i][j].value[iday][ihour] = rainAllCell[i*ROFnum_col + j];
				}
			}
		}
	}

	// read one month data. 1̎ԃf[^ǂݍ
/*	float rain;
	for(int iday=1; iday <= dayMonth[n_month-1]; iday++){  // day loop
		for(int ihour=0; ihour<24; ihour++){  // hour loop
			for(int i=0; i<ROFnum_row; i++){  // 1st row data is the southest row!
				for(int j=0; j<ROFnum_col; j++){  // 1st column is the westest column!
					fin.read((char*)&rain, sizeof(float));
					fROF[i][j].value[iday][ihour] = rain;
				}
			}
		}
	}
*/
	
	fin.close();
}


void read2DValueAsc(ROF** fROF, int n_year, int n_month, int ROFnum_row, int ROFnum_col)
{
	// read ascii format two dimensional data. 

	char ROFfilename[256];

	//sprintf_s(ROFfilename, 256, "./input/%04d%02d_rain.dat", n_year, n_month);
	sprintf(ROFfilename, "./input/%04d%02d_rain.dat", n_year, n_month);
	cout << ROFfilename << endl;

	ifstream fin(ROFfilename);
	if(!fin){
		cout << "ROF file cannot open.\n";
		exit(1);
	}

	//Ǒ̖̃f[^ value[0][ihour] ɃZbg
	const int dayMonth[] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};	

	int c_month;
	c_month = n_month - 1;
	if(c_month == 0){
		c_month = c_month + 12;
	}

	// O̍ŏI̎ԗʂ0ڂ̎ԗʂɃZbg
	for(int i=0; i<ROFnum_row; i++){
		for(int j=0; j<ROFnum_col; j++){		
			for(int ihour=0; ihour<24; ihour++){
				fROF[i][j].value[0][ihour] = fROF[i][j].value[dayMonth[c_month-1]][ihour];
			}
		}
	}

	// read one month data. 1̎ԃf[^ǂݍ
	int year, month, day, hour, xcol, yrow;
	for(int iday=1; iday <= dayMonth[n_month-1]; iday++){  // day loop
		for(int ihour=0; ihour<24; ihour++){ // hour loop
			fin >> year >> month >> day >> hour >> xcol >> yrow; // read header for each hour
			for(int i=ROFnum_row-1; i>=0; i--){ // 1st row data is the northest row!
				for(int j=0; j<ROFnum_col; j++){ // 1st column is the westest column!
					fin >> fROF[i][j].value[iday][ihour] ;
				}
			}
		}
	}

	fin.close();
}

